"""Build beta detail models for the API"""

import enum
from typing import Dict, Optional

import deserialize

from asconnect.models.common import Links, Relationship, Resource


class ExternalBetaState(enum.Enum):
    """External beta state."""

    processing = "PROCESSING"
    processing_exception = "PROCESSING_EXCEPTION"
    missing_export_compliance = "MISSING_EXPORT_COMPLIANCE"
    ready_for_beta_testing = "READY_FOR_BETA_TESTING"
    in_beta_testing = "IN_BETA_TESTING"
    expired = "EXPIRED"
    ready_for_beta_submission = "READY_FOR_BETA_SUBMISSION"
    in_export_compliance_review = "IN_EXPORT_COMPLIANCE_REVIEW"
    waiting_for_beta_review = "WAITING_FOR_BETA_REVIEW"
    in_beta_review = "IN_BETA_REVIEW"
    beta_rejected = "BETA_REJECTED"
    beta_approved = "BETA_APPROVED"


class InternalBetaState(enum.Enum):
    """Internal beta state."""

    processing = "PROCESSING"
    processing_exception = "PROCESSING_EXCEPTION"
    missing_export_compliance = "MISSING_EXPORT_COMPLIANCE"
    ready_for_beta_testing = "READY_FOR_BETA_TESTING"
    in_beta_testing = "IN_BETA_TESTING"
    expired = "EXPIRED"
    in_export_compliance_review = "IN_EXPORT_COMPLIANCE_REVIEW"


@deserialize.key("identifier", "id")
class BuildBetaDetail(Resource):
    """Represents a build localization."""

    @deserialize.key("auto_notify_enabled", "autoNotifyEnabled")
    @deserialize.key("external_build_state", "externalBuildState")
    @deserialize.key("internal_build_state", "internalBuildState")
    class Attributes:
        """Represents beta build localization attributes."""

        auto_notify_enabled: bool
        external_build_state: ExternalBetaState
        internal_build_state: InternalBetaState

    identifier: str
    attributes: Attributes
    relationships: Optional[Dict[str, Relationship]]
    links: Links



"""App Models for the API"""

import enum
from typing import Dict, Optional

import deserialize

from asconnect.models.common import Resource, Links, Relationship


class Platform(enum.Enum):
    """The different platforms an app can be for."""

    ios = "IOS"
    macos = "MAC_OS"
    tvos = "TV_OS"


class ReleaseType(enum.Enum):
    """App store release type."""

    manual = "MANUAL"
    after_approval = "AFTER_APPROVAL"
    scheduled = "SCHEDULED"


class AppStoreVersionState(enum.Enum):
    """App store version state."""

    developer_removed_from_sale = "DEVELOPER_REMOVED_FROM_SALE"
    developer_rejected = "DEVELOPER_REJECTED"
    in_review = "IN_REVIEW"
    invalid_binary = "INVALID_BINARY"
    metadata_rejected = "METADATA_REJECTED"
    pending_apple_release = "PENDING_APPLE_RELEASE"
    pending_contract = "PENDING_CONTRACT"
    pending_developer_release = "PENDING_DEVELOPER_RELEASE"
    prepare_for_submission = "PREPARE_FOR_SUBMISSION"
    preorder_ready_for_sale = "PREORDER_READY_FOR_SALE"
    processing_for_app_store = "PROCESSING_FOR_APP_STORE"
    ready_for_sale = "READY_FOR_SALE"
    rejected = "REJECTED"
    removed_from_sale = "REMOVED_FROM_SALE"
    waiting_for_export_compliance = "WAITING_FOR_EXPORT_COMPLIANCE"
    waiting_for_review = "WAITING_FOR_REVIEW"
    replaced_with_new_version = "REPLACED_WITH_NEW_VERSION"


@deserialize.key("identifier", "id")
class AppStoreVersion(Resource):
    """Represents an app store version."""

    @deserialize.key("app_store_state", "appStoreState")
    @deserialize.key("earliest_release_date", "earliestReleaseDate")
    @deserialize.key("release_type", "releaseType")
    @deserialize.key("uses_idfa", "usesIdfa")
    @deserialize.key("version_string", "versionString")
    @deserialize.key("created_date", "createdDate")
    class Attributes:
        """Attributes."""

        platform: Platform
        app_store_state: AppStoreVersionState
        copyright: str
        earliest_release_date: Optional[str]
        release_type: ReleaseType
        uses_idfa: Optional[bool]
        version_string: str
        created_date: str
        downloadable: bool

    identifier: str
    attributes: Attributes
    relationships: Optional[Dict[str, Relationship]]
    links: Links


@deserialize.key("identifier", "id")
class AppStoreReviewDetails(Resource):
    """Represents an app store review details."""

    @deserialize.key("contact_email", "contactEmail")
    @deserialize.key("contact_first_name", "contactFirstName")
    @deserialize.key("contact_last_name", "contactLastName")
    @deserialize.key("contact_phone", "contactPhone")
    @deserialize.key("demo_account_name", "demoAccountName")
    @deserialize.key("demo_account_password", "demoAccountPassword")
    @deserialize.key("demo_account_required", "demoAccountRequired")
    class Attributes:
        """Attributes."""

        contact_email: str
        contact_first_name: str
        contact_last_name: str
        contact_phone: str
        demo_account_name: str
        demo_account_password: str
        demo_account_required: bool
        notes: str

    identifier: str
    attributes: Attributes
    relationships: Optional[Dict[str, Relationship]]
    links: Links



"""Localization models for the API"""

import enum
from typing import Dict, Optional

import deserialize

from asconnect.models.common import Links, Relationship, Resource
from asconnect.models.app_store import AppStoreVersionState


@deserialize.key("identifier", "id")
class AppInfoLocalization(Resource):
    """Represents a build."""

    @deserialize.key("privacy_policy_text", "privacyPolicyText")
    @deserialize.key("privacy_policy_url", "privacyPolicyUrl")
    class Attributes:
        """Represents app info localization attributes."""

        locale: Optional[str]
        name: Optional[str]
        privacy_policy_text: Optional[str]
        privacy_policy_url: Optional[str]
        subtitle: Optional[str]

    identifier: str
    attributes: Attributes
    relationships: Optional[Dict[str, Relationship]]
    links: Links


class AppStoreAgeRating(enum.Enum):
    """App store age rating."""

    four_plus = "FOUR_PLUS"
    nine_plus = "NINE_PLUS"
    twelve_plus = "TWELVE_PLUS"
    seventeen_plus = "SEVENTEEN_PLUS"


class BrazilAgeRating(enum.Enum):
    """Brazil age rating."""

    l = "L"
    ten = "TEN"
    twelve = "TWELVE"
    fourteen = "FOURTEEN"
    sixteen = "SIXTEEN"
    eighteen = "EIGHTEEN"


class KidsAgeBand(enum.Enum):
    """Kids age band."""

    five_and_under = "FIVE_AND_UNDER"
    six_to_eight = "SIX_TO_EIGHT"
    nine_to_eleven = "NINE_TO_ELEVEN"


@deserialize.key("identifier", "id")
class AppInfo(Resource):
    """Represents an apps info."""

    @deserialize.key("app_store_age_rating", "appStoreAgeRating")
    @deserialize.key("app_store_state", "appStoreState")
    @deserialize.key("brazil_age_rating", "brazilAgeRating")
    @deserialize.key("kids_age_band", "kidsAgeBand")
    class Attributes:
        """Represents app info localization attributes."""

        app_store_age_rating: AppStoreAgeRating
        app_store_state: AppStoreVersionState
        brazil_age_rating: BrazilAgeRating
        kids_age_band: Optional[KidsAgeBand]

    identifier: str
    attributes: Attributes
    relationships: Optional[Dict[str, Relationship]]
    links: Links



"""Models for the API"""

from typing import Optional

import deserialize


@deserialize.key("self_link", "self")
class Links:
    """Represents item links."""

    self_link: Optional[str]
    first: Optional[str]
    next: Optional[str]
    related: Optional[str]


class Relationship:
    """Represents a relationship."""

    links: Links


class Paging:
    """Represents a paging data item in a REST response."""

    total: int
    limit: int


class Meta:
    """Represents a meta data item in a REST response."""

    paging: Paging


@deserialize.key("resource_type", "type")
class Resource:
    """Represents a resource."""

    resource_type: str



"""Wrapper for altool."""

import enum
import logging
import subprocess
from typing import Optional


class Platform(enum.Enum):
    """The platforms that altool supports."""

    ios = "ios"
    macos = "osx"
    tvos = "appletvos"


def upload(
    *,
    ipa_path: str,
    platform: Platform,
    key_id: str,
    issuer_id: str,
    log: Optional[logging.Logger] = None,
) -> None:
    """Upload a build to app store connect.

    :param ipa_path: The path to the ipa to upload
    :param platform: The platform the app is for
    :param key_id: The ID for the key used for auth
    :param issuer_id: The ID of the issuer of the auth key
    :param log: Any base logger to be used (one will be created if not supplied)

    :raises CalledProcessError: If something goes wrong during upload
    """

    if log:
        log = log.getChild(__name__)
    else:
        log = logging.getLogger(__name__)

    command = [
        "xcrun",
        "altool",
        "--upload-app",
        "-f",
        ipa_path,
        "-t",
        platform.value,
        "-apiKey",
        key_id,
        "--apiIssuer",
        issuer_id,
    ]

    log.info(
        "Beginning upload. This can take a while and will not show any output while in progress."
    )

    upload_process = subprocess.Popen(
        command,
        universal_newlines=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
    )

    assert upload_process.stdout is not None
    command_output = ""
    for line in iter(upload_process.stdout.readline, ""):
        log.info(line.rstrip())
        command_output += line
    upload_process.stdout.close()
    upload_process.wait()

    if upload_process.returncode != 0:
        raise subprocess.CalledProcessError(
            upload_process.returncode,
            command,
            "Failed to upload the build. Please see the logs for more information.",
        )



"""Sorting options."""

import enum


class BuildsSort(enum.Enum):
    """Orders that builds can be sorted."""

    PreReleaseVersion = "preReleaseVersion"
    PreReleaseVersionReversed = "-preReleaseVersion"
    UploadedDate = "uploadedDate"
    UploadedDateReversed = "-uploadedDate"
    Version = "version"
    VersionReversed = "-version"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['asconnect', 'asconnect.models']

package_data = \
{'': ['*']}

install_requires = \
['cryptography>=3.0.0,<4.0.0',
 'deserialize>=1.2.0,<2.0.0',
 'pyjwt>=1.7.0,<2.0.0',
 'requests>=2.20.0,<3.0.0']

setup_kwargs = {
    'name': 'asconnect',
    'version': '2.0.1',
    'description': 'A wrapper around the Apple App Store Connect APIs',
    'long_description': '# asconnect\n\nasconnect is a Python wrapper around the [Apple App Store Connect REST APIs](https://developer.apple.com/documentation/appstoreconnectapi).\n\nThis wrapper does not cover every API, but does cover the basics, including:\n\n* Uploading a build\n* Creating a new TestFlight version\n* Setting TestFlight review information\n* Creating a new app store version\n* Setting the app review information\n* Submitting for app review\n\n## Getting Started\n\n### Installation\n\nThe package is available on PyPI, so you can run `pip install asconnect` to get the latest version.\n\n### Creating a client\n\nTo begin, you need to [generate a key](https://developer.apple.com/documentation/appstoreconnectapi/creating_api_keys_for_app_store_connect_api), then get it\'s ID, the contents of the key itself, and the issuer ID.\n\nOnce you have those, you can create a new client by running:\n\n```python\nclient = asconnect.Client(key_id="...", key_contents="...", issuer_id="...")\n```\n\n### Getting your App\n\nMost operations require an app identifier. This is not the same as the bundle ID you choose, but is an ID generated by Apple. The easiest way to get this is to run this code:\n\n```python\napp = client.app.get_from_bundle_id("com.example.my_bundle_id")\n```\n\n### Uploading a Build\n\nUploading a build isn\'t technically part of the App Store Connect APIs, but a wrapper around altool is included to make things as easy as possible. Let\'s upload a build for your app:\n\n```python\nclient.build.upload(\n  ipa_path="/path/to/the/app.ipa",\n  platform=asconnect.Platform.ios,\n)\n```\n\nAnd if you want to wait for your build to finish processing:\n\n```python\nbuild = client.build.wait_for_build_to_process("com.example.my_bundle_id", build_number)\n```\n\n`build_number` is the build number you gave your build when you created it. It\'s used by the app store to identify the build.\n\n### App Store Submission\n\nLet\'s take that build, create a new app store version and submit it,\n\n```python\n# Create a new version\nversion = client.app.create_new_version(version="1.2.3", app_id=app.identifier)\n\n# Set the build for that version\nclient.version.set_build(version_id=version.identifier, build_id=build.identifier)\n\n# Submit for review\nclient.version.submit_for_review(version_id=version.identifier)\n```\n\nIt\'s that easy. Most of the time at least. If you don\'t have previous version to inherit information from you\'ll need to do things like set screenshots, reviewer info, etc. All of which is possible through this library.\n\n\n# Contributing\n\nThis project welcomes contributions and suggestions.  Most contributions require you to agree to a\nContributor License Agreement (CLA) declaring that you have the right to, and actually do, grant us\nthe rights to use your contribution. For details, visit https://cla.opensource.microsoft.com.\n\nWhen you submit a pull request, a CLA bot will automatically determine whether you need to provide\na CLA and decorate the PR appropriately (e.g., status check, comment). Simply follow the instructions\nprovided by the bot. You will only need to do this once across all repos using our CLA.\n\nThis project has adopted the [Microsoft Open Source Code of Conduct](https://opensource.microsoft.com/codeofconduct/).\nFor more information see the [Code of Conduct FAQ](https://opensource.microsoft.com/codeofconduct/faq/) or\ncontact [opencode@microsoft.com](mailto:opencode@microsoft.com) with any additional questions or comments.\n',
    'author': 'Dale Myers',
    'author_email': 'dalemy@microsoft.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/microsoft/asconnect',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
